#Packages 
using LinearAlgebra
using Plots
using FFTW

#%Complex definition
i=sqrt(Complex(-1));

#Definition of spin system
include("basis.jl")
nspins=6
iu,ix,iy,iz,ip,im,ia,ib=basis(nspins)
cs=[-450 10 10 450 450 450 0 0 0 0];
sc=[0 1 1 0 0 0 0 0 0 0;
    0 0 0 7 7 7 0 0 0 0;
    0 0 0 7 7 7 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0];
dc=[0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0 0];

#simulation parameters
dwell=0.001;
T2_time=1.0;
#npoints=1024;
npoints=8192;
tp=0.001/4

#array definitions
fid=zeros(ComplexF64,npoints)
time=zeros(npoints)

#spin densities, Hamiltonians, propagators
fm = zeros(2^nspins,2^nspins)
fp = zeros(2^nspins,2^nspins)
fx = zeros(2^nspins,2^nspins)
fy = zeros(2^nspins,2^nspins)
fz = zeros(2^nspins,2^nspins)
hcs = zeros(2^nspins,2^nspins)
hsc = zeros(2^nspins,2^nspins)
hdc = zeros(2^nspins,2^nspins)
for ispins = 1:nspins
    global fm = fm + im[:,:,ispins];
    global fp = fp + ip[:,:,ispins];
    global fx = fx + ix[:,:,ispins];
    global fy = fy + iy[:,:,ispins];
    global fz = fz + iz[:,:,ispins];
    global hcs = hcs - 2*pi*cs[ispins]*iz[:,:,ispins];
    if ispins < nspins
        for jspins = ispins+1:nspins
            global hsc = hsc + 2*pi*sc[ispins,jspins]*(ix[:,:,ispins]*ix[:,:,jspins]+iy[:,:,ispins]*iy[:,:,jspins]+iz[:,:,ispins]*iz[:,:,jspins])
            global hdc = hdc + 2*pi*dc[ispins,jspins]*(-ix[:,:,ispins]*ix[:,:,jspins]-iy[:,:,ispins]*iy[:,:,jspins]+2*iz[:,:,ispins]*iz[:,:,jspins])
            println("sc,dc[",ispins,jspins,"]: ",sc[ispins,jspins]," [Hz] , ", dc[ispins,jspins]," [Hz]")
        end
    end
end

rhoinit=fz

hcs1 = -2*pi*(-450)*iz[:,:,1]
hcs2 = -2*pi*(250)*iz[:,:,2]
hcs3 = -2*pi*(20)*iz[:,:,3]
hj12 = 2*pi*10*(ix[:,:,1]*ix[:,:,2]+iy[:,:,1]*iy[:,:,2]+iz[:,:,1]*iz[:,:,2])
hj13 = 2*pi*20*(ix[:,:,1]*ix[:,:,3]+iy[:,:,1]*iy[:,:,3]+iz[:,:,1]*iz[:,:,3])
hj23 = 2*pi*30*(ix[:,:,2]*ix[:,:,3]+iy[:,:,2]*iy[:,:,3]+iz[:,:,2]*iz[:,:,3])
#hevo = hcs1+hcs2+hj12
hevo = hcs+hsc+hdc
uevo = exp(-i*hevo*dwell)

#ham90y= 2*pi*1000*(iy[:,:,1]+iy[:,:,2])
#u90y = exp(-i*(ham90y+hevo)*0.001/4)
u90y = exp(-i*2*pi*fy/4)

#simulating the FID
rho = rhoinit
rho = u90y*rho*u90y'
for ipoints=1:npoints
    time[ipoints]=(ipoints-1)*dwell
    fid[ipoints]=tr(rho*fm)*exp(-(ipoints-1)*dwell/T2_time)
    global rho = uevo*rho*uevo'
end

#plot FID / Fourier-transformed spectrum
plot(time,real(fid), thickness_scaling=2, legend=false, GradientDescent=false)
plot(fftshift(fftfreq(npoints,1/dwell)),real(fftshift(fft(fid))))

