
#Packages 
using LinearAlgebra
using Plots
using FFTW

#Choose exact or fast approximate Calculation
exact = true
exact = false

#%Complex definition
i=sqrt(Complex(-1));

#Definition of individual spin operators
nspins=1
iu,ix,iy,iz,ip,im,ia,ib=basis(nspins)

#Define RF_max and pulse length for Bruker pulse
rfmax=3069
tpulse=2000/1000000

#Read in Bruker wave file
#filename="C:/Users/bulu/JULIA/SPINSYSTEMS/xyBEBOP_190u_RF10kHz_BW200kHz"
#filename="C:/Users/bulu/JULIA/SPINSYSTEMS/test_input"
#filename="C:/Bruker/TopSpin4.2.0/exp/stan/nmr/lists/wave/Burbop-180.1"
filename="C:/Bruker/TopSpin4.2.0/exp/stan/nmr/lists/wave/Reburp.1000"

ampl,phase=read_bruker_wave(filename)
ampl=ampl*rfmax/100.0
tgrid=collect(tpulse/size(ampl)[1]:tpulse/size(ampl)[1]:tpulse)
tdigit=tpulse/size(ampl)[1]

#simulation parameters
noffs=256
width=10000

#array definitions
offs=zeros(noffs)
mrho=zeros(ComplexF64,2,2,noffs)
uoffs=zeros(ComplexF64,2,2,noffs)
Mx=zeros(noffs)
My=zeros(noffs)
Mz=zeros(noffs)

#spin densities, Hamiltonians, propagators
rhoinit=iz[:,:,1]#+iz[:,:,2]
norm=real(sum(ix[:,:,1].*conj.(ix[:,:,1])))

#first exact, later approximate fast calculation
if exact

#simulate offset dependence of pulse/pulse sequence
for ioffs=1:noffs
    hoffs = 2*pi*(-width/2+ioffs*width/noffs)*iz[:,:,1]

    global rho = rhoinit
    for ip = 1:size(ampl)[1]
        hamp=2*pi*ampl[ip]*(cosd(phase[ip])*ix[:,:,1]+sind(phase[ip])*iy[:,:,1])
        local uevo=exp(-i*(hamp+hoffs)*tdigit)
        global rho = uevo*rho*uevo'
    end
    offs[ioffs]=(-width/2+ioffs*width/noffs)
    Mx[ioffs]=real(sum(rho.*conj.(ix[:,:,1])))/norm
    My[ioffs]=real(sum(rho.*conj.(iy[:,:,1])))/norm
    Mz[ioffs]=real(sum(rho.*conj.(iz[:,:,1])))/norm
    if ioffs==128 @show(rho,iz,sum(rho.*conj.(iz[:,:,1]))) end
#    Mx[ioffs]=real(tr(rho*ix[:,:,1]))
#    My[ioffs]=real(tr(rho*iy[:,:,1]))
#    Mz[ioffs]=real(tr(rho*iz[:,:,1]))
end

else

#simulate offset dependence of pulse/pulse sequence
for ioffs=1:noffs
    hoffs = 2*pi*(-width/2+ioffs*width/noffs)*iz[:,:,1]
    global uoffs[:,:,ioffs] = exp(-i*hoffs*tdigit)
    global mrho[:,:,ioffs]=rhoinit
end    
for ip = 1:size(ampl)[1]
    hamp=2*pi*ampl[ip]*(cosd(phase[ip])*ix[:,:,1]+sind(phase[ip])*iy[:,:,1])
    local uevo=exp(-i*(hamp)*tdigit)
    for ioffs=1:noffs
        global mrho[:,:,ioffs] = uoffs[:,:,ioffs]*uevo*mrho[:,:,ioffs]*uevo'*uoffs[:,:,ioffs]'
    end
end
for ioffs=1:noffs
    offs[ioffs]=(-width/2+ioffs*width/noffs)
    Mx[ioffs]=real(sum(mrho[:,:,ioffs].*conj.(ix[:,:,1])))/norm
    My[ioffs]=real(sum(mrho[:,:,ioffs].*conj.(iy[:,:,1])))/norm
    Mz[ioffs]=real(sum(mrho[:,:,ioffs].*conj.(iz[:,:,1])))/norm
end

end
#offset plots of final magnetization components
plot(offs,[Mx,My,Mz,sqrt.(Mx.^2+My.^2)],label=["Mx" "My" "Mz" "Mxy"])

#plot(tgrid,[ampl,phase],label=["rf-amplitude" "xy-Phase"] )
